"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Url = require('url-parse');
var request = require('request');
var crypto = require('crypto');
var jobQueue = require('./jobQueue');
var queueNames = require('./queueNames');

var Base = function () {
  function Base(request, config, appName) {
    _classCallCheck(this, Base);

    this.request = request;
    this.config = config;
    this.appName = this.getValidAppName(appName);
    this.apiSecret = config.communicator.signingKey;
    this.queueNames = queueNames;
    this.domain = null;
    this.queue = jobQueue.PractoQueue.getInstance(config.communicator.communicatorQueueUri, config.communicator.sqsSpoolDir);
    try {
      if (request.headers.host.indexOf('http') === -1) {
        if (request.headers.host.indexOf('.local') > -1) {
          this.domain = 'http://' + request.headers.host;
        } else {
          this.domain = 'https://' + request.headers.host;
        }
      } else {
        this.domain = request.headers.host;
      }
      this.queue.setDefaultHost(this.domain);
    } catch (err) {}
  }

  _createClass(Base, [{
    key: 'setDomain',
    value: function setDomain() {
      var domain = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

      this.domain = domain;
      this.queue.setDefaultHost(this.domain);
    }
  }, {
    key: 'getDomain',
    value: function getDomain() {
      return this.domain;
    }
  }, {
    key: 'getCommunicatorUri',
    value: function getCommunicatorUri() {
      domain = this.domain;
      var parts = new Url(domain);
      var host = parts.hostname;
      var hostParts = host.split('.', 1);
      var subDomain = hostParts[0];
      subDomain = subDomain.split('-');
      var uri = parts.protocol + '//communicator';
      if (subDomain.length > 1) {
        uri = uri + '-' + subDomain[1];
      }
      uri = uri + '.' + host.substr(host.indexOf('.') + 1);
      return uri;
    }
  }, {
    key: 'generateNonce',
    value: function generateNonce() {
      var currentTime = +new Date() / 1000;
      var base = "abcdefghijklmnopqrstuvwxyz0123456789";
      var ret = parseInt(currentTime).toString() + '|';
      for (var i = 0; i < 20; i++) {
        ret = ret + base[Math.floor(Math.random() * (base.length - 0) + 0)];
      }
      return ret;
    }
  }, {
    key: 'get',
    value: function get(endPoint, requestParams, callback) {
      var nonce = this.generateNonce();
      var authToken = crypto.createHash('sha256').update(this.apiSecret + nonce.toString()).digest('hex');
      requestParams.appName = this.appName;
      var apiBaseUrl = this.getCommunicatorUri();
      var apiUrl = apiBaseUrl + endPoint.toString();
      var options = {
        url: apiUrl,
        qs: requestParams,
        method: 'GET',
        headers: {
          'nonce': nonce,
          'X-AUTH-TOKEN': authToken
        }
      };
      request(options, function (err, response, body) {
        if (err) {
          callback(err);
        } else {
          callback(null, response);
        }
      });
    }
  }, {
    key: 'post',
    value: function post(endPoint, requestParams, callback) {
      var nonce = this.generateNonce();
      var authToken = crypto.createHash('sha256').update(this.apiSecret + nonce.toString()).digest('hex');
      requestParams.appName = this.appName;
      var apiBaseUrl = this.getCommunicatorUri();
      var apiUrl = apiBaseUrl + endPoint.toString();
      var options = {
        url: apiUrl,
        body: requestParams,
        method: 'POST',
        headers: {
          'nonce': nonce,
          'X-AUTH-TOKEN': authToken
        }
      };
      request(options, function (err, response, body) {
        if (err) {
          callback(err);
        } else {
          callback(null, response);
        }
      });
    }
  }, {
    key: 'send',
    value: function send(data, queueName, delay) {
      this.queue.insertJob(JSON.stringify(data), queueName, delay = delay);
    }
  }, {
    key: 'getValidAppName',
    value: function getValidAppName(appName) {
      var apps = ["RAY", "HELLO", "FABRIC", "EPICENTER", "ACCOUNTS", "STORE", "DX-NEO", "HOLONET", "GENESIS", "ONENESS", "TODO", "ADDON"];
      appName = appName.toUpperCase();
      if (apps.indexOf(appName) > -1) {
        return appName;
      } else {
        throw "Invalid application name. Has to be one of \
            RAY, HELLO, FABRIC, EPICENTER, ACCOUNTS, STORE, \
            DX-NEO, HOLONET, GENESIS, ONENESS";
      }
    }
  }]);

  return Base;
}();

module.exports = {
  Base: Base
};