"use strict"

let base =  require('./base');
let types = require('./types');

class SmsGenericPromotional extends base.Base{
  constructor(request, config, appName){
    super(request, config, appName);
  }

  sendSms(referenceId, referenceType, name, to, messageType,
           body, initiatedAt, expiryTime=null, senderId=null,
           campaignId=null){
    let job = new types.GenericPromotionalSms();
    job.reference_id = referenceId;
    job.reference_type = referenceType;
    job.name = name;
    job.mobile = to;
    job.app_name = this.appName;
    job.message_type = messageType.toUpperCase();
    job.message_text = body;
    job.initiated_at = body;
    job.expiry_time = expiryTime;
    job.sender_id = senderId;
    job.campaign_id = campaignId;
    job.sms_type = 'generic_promotional';
    this.send(job, this.queueNames.sendGenericPromotionalSmsQueue, 0);
  }

  getMessagesByReferenceId(referenceId, smsTypes, callback){
    if(!Array.isArray(smsTypes)){
      throw "smsTypes should be array";
    }
    let data = {};
    data.reference_id = referenceId;
    data.sms_types = JSON.stringify(smsTypes);
    this.get(
      '/genericpromotionalsms/get_messages_by_reference_id', data, (err, response) =>{
        if(err){
          callback(err);
        }
        else{
          callback(null, response);
        }
      });
  }

  isMessageSentOfTypeForRecipient(messageType, mobile, callback){
    let data = {};
    data.message_type = messageType;
    data.mobile = mobile;
    this.get(
      '/genericpromotionalsms/is_msg_sent_of_type_for_recipient', data, (err, response)=>{
        if(err){
          callback(err);
        }
        else{
          callback(null, response);
        }
      });
  }

  getMessageCountByType(messageType, startDate, endDate, status, callback){
    let data = {};
    data.message_type = messageType;
    data.start_date = startDate;
    data.end_date = endDate;
    data.status = status;
    const response = this.get(
      '/genericpromotionalsms/get_message_count_for_type', data, (err, response)=>{
        if(err){
          callback(err);
        }
        else{
          callback(null, response);
        }
      });
  }

  getMessageCountByReferenceIds(referenceIds, smsTypes, callback){
    if(!Array.isArray(referenceIds)){
      throw "referenceIds should be array";
    }
    if(!Array.isArray(smsTypes)){
      throw "smsTypes should be array";
    }
    let data = {};
    data.reference_ids = JSON.stringify(referenceIds);
    data.sms_types = JSON.stringify(smsTypes);
    this.get(
      '/genericpromotionalsms/get_message_count_by_reference_ids', data, (err, response)=>{
        if(err){
          callback(err);
        }
        else{
          callback(null, response);
        }
      });
  }

  validateSmsType(type){
    const allowedTypes = [
      'VN_APP_PROMOTION_SMS',
      'FIT_RAY_SHARE_SMS',
      'PHR_TAX_SAVING_NOTIFICATION_SMS'
    ];

    if(allowedTypes.indexOf(type) < 0){
      throw "Invalid mail type";
    }
  }
}

module.exports = {
  SmsGenericPromotional: SmsGenericPromotional
}
