"use strict"

let base =  require('./base');
let types = require('./types');
let DateTime = require('node-datetime');

class OtpSms extends base.Base{
  constructor(request, config, appName){
    super(request, config, appName);
  }

  createOtpJob(transactionId, transactionType, to, otp,
               maxRetry, expiryTime, isNewTemplate=false){
    let job = new types.Otp();
    job.transaction_id = transactionId;
    job.transaction_type = transactionType;
    job.mobile = to;
    job.app_name = this.appName;
    job.otp = otp;
    job.expiry_time = expiryTime;
    job.max_retry = maxRetry;
    job.is_new_template = isNewTemplate;
    this.validate(job);
    return job;
  }

  validate(job){
    if(!job.otp){
      throw "OTP is missing";
    }
    if(!job.transaction_id){
      throw "transactionId is missing";
    }
    if(!job.transaction_yype){
      throw "transactionType is missing";
    }
    if(!job.mobile){
      throw "mobile is missing";
    }
    if(!job.app_name){
      throw "appName is missing";
    }
    if(!job.max_retry){
      throw "maxRetry is missing";
    }
    if(job.max_retry > 10){
      throw "Max retry should be less than 10";
    }
  }

  getTransactionId(){
    let currentTime = (+new Date)/1000;
    const base = "abcdefghijklmnopqrstuvwxyz0123456789";
    let ret = parseInt(currentTime).toString() + '|';
    for(let i=0; i<20; i++){
      ret = ret + base[Math.floor(Math.random() * (base.length - 0) + 0)];
    }
    return ret;
  }

  sendCall(transactionId, transactionType, to, otp, maxRetry=10, expiryTime=null){
    const job = this.createOtpJob(transactionId, transactionType, to, otp,
      maxRetry, expiryTime);
    this.send(job, this.queue.sendCallOtpQueue, 0);
  }

  sendSms(transactionId, transactionType, to, otp, maxRetry=10, expiryTime=null,
    isNewTemplate=false){
    const job = this.createOtpJob(transactionId, transactionType, to, otp,
      maxRetry, expiryTime, isNewTemplate);
    this.send(job, this.queue.sendSmsOtpQueue, 0);
  }

  completeOtpTransaction(transactionId, mobile, otp){
    let job = {};
    job.transaction_id = transactionId;
    job.mobile = mobile;
    job.otp = otp;
    job.completed_at = DateTime.create().format('Y-M-d H:M:S');
    this.send(job, this.queue.completeOtpTransaction, 0);
  }
}

module.exports = {
  OtpSms: OtpSms
}
